# -*- coding: utf-8 -*-
import pytest

from functools import partial

from click.testing import CliRunner
import ruamel.yaml

from trailblazer.cli import base
from trailblazer.mip import sacct, files as files_api
from trailblazer.log import LogAnalysis
from trailblazer.store import Store


@pytest.fixture
def cli_runner():
    runner = CliRunner()
    return runner


@pytest.fixture
def invoke_cli(cli_runner):
    return partial(cli_runner.invoke, base)


@pytest.fixture(scope='session')
def files():
    return {
        'config': 'tests/fixtures/family/family_config.yaml',
        'sampleinfo': 'tests/fixtures/family/family_qc_sample_info.yaml',
        'qcmetrics': 'tests/fixtures/family/family_qc_metrics.yaml',
        'sacct': 'tests/fixtures/family/mip.pl_2017-06-17T12:11:42.log.status',
    }


@pytest.fixture(scope='session')
def files_raw(files):
    return {
        'config': ruamel.yaml.safe_load(open(files['config'])),
        'sampleinfo': ruamel.yaml.safe_load(open(files['sampleinfo'])),
        'qcmetrics': ruamel.yaml.safe_load(open(files['qcmetrics'])),
        'sacct': list(open(files['sacct'])),
    }


@pytest.fixture(scope='session')
def files_data(files_raw):
    return {
        'config': files_api.parse_config(files_raw['config']),
        'sampleinfo': files_api.parse_sampleinfo(files_raw['sampleinfo']),
        'qcmetrics': files_api.parse_qcmetrics(files_raw['qcmetrics']),
        'sacct': sacct.parse_sacct(files_raw['sacct']),
    }


@pytest.fixture(scope='session')
def allfailed_sacct_jobs():
    with open('tests/fixtures/sacct/allfailed.log.status') as stream:
        sacct_jobs = sacct.parse_sacct(stream)
    return sacct_jobs


@pytest.yield_fixture(scope='function')
def store(tmpdir):
    _store = Store(uri='sqlite://', families_dir=tmpdir)
    _store.setup()
    yield _store
    _store.drop_all()


@pytest.yield_fixture(scope='function')
def sample_store(store):
    sample_data = ruamel.yaml.safe_load(open('tests/fixtures/sample-data.yaml'))
    for user_data in sample_data['users']:
        store.add_user(user_data['name'], user_data['email'])
    for analysis_data in sample_data['analyses']:
        analysis_data['user'] = store.user(analysis_data['user'])
        failed_jobs = analysis_data.get('failed_jobs', [])
        analysis_data['failed_jobs'] = [store.Job(**job_data) for job_data in failed_jobs]
        store.add(store.Analysis(**analysis_data))
    store.commit()
    store.add_pending(family='gentlebird', email='tom.cruise@magnolia.com')
    yield store


@pytest.fixture
def log_analysis(store):
    _log_analysis = LogAnalysis(store)
    return _log_analysis



# -*- coding: utf-8 -*-


class TrailblazerError(Exception):

    def __init__(self, message):
        self.message = message


class MissingFileError(TrailblazerError):
    pass


class MipStartError(TrailblazerError):
    pass


class ConfigError(TrailblazerError):

    def __init__(self, message, errors=None):
        self.message = message
        self.errors = errors



# -*- coding: utf-8 -*-
import logging
from pathlib import Path
from typing import List

import ruamel.yaml

from trailblazer.mip import sacct as sacct_api, files as files_api, miplog
from trailblazer.store import models, Store
from trailblazer.exc import MissingFileError

log = logging.getLogger(__name__)


class LogAnalysis(object):

    def __init__(self, store: Store):
        self.store = store

    def __call__(self, config_stream: List[str], sampleinfo: str=None, sacct: str=None):
        """Add a new analysis log."""
        config_raw = ruamel.yaml.safe_load(config_stream)
        config_data = files_api.parse_config(config_raw)
        sampleinfo_path = Path(sampleinfo or config_data['sampleinfo_path'])
        if not sampleinfo_path.exists():
            raise MissingFileError(sampleinfo_path)
        with sampleinfo_path.open() as stream:
            sampleinfo_raw = ruamel.yaml.safe_load(stream)
        sampleinfo_data = files_api.parse_sampleinfo(sampleinfo_raw)
        sacct_path = Path(sacct if sacct else f"{config_data['log_path']}.status")
        if not sacct_path.exists():
            raise MissingFileError(sacct_path)
        with sacct_path.open() as stream:
            sacct_jobs = sacct_api.parse_sacct(stream)
        with Path(config_data['log_path']).open() as stream:
            jobs = len(miplog.job_ids(stream))
        run_data = self.parse(config_data, sampleinfo_data, sacct_jobs, jobs=jobs)
        self._delete_temp_logs(run_data['family'])
        new_run = self.build(run_data)
        if new_run:
            self.store.add_commit(new_run)
            return new_run

    def _delete_temp_logs(self, family_name: str):
        """Delete temporary logs for the current family."""
        for temp_log in self.store.analyses(family=family_name, temp=True):
            log.debug(f"delete temporary log: {temp_log.id} - {temp_log.status}")
            temp_log.delete()

    @classmethod
    def parse(cls, config_data: dict, sampleinfo_data: dict, sacct_jobs: List[dict],
              jobs: int) -> dict:
        """Parse information about a run."""
        analysis_types = [sample['type'] for sample in config_data['samples']]
        run_data = {
            'user': config_data['email'],
            'family': config_data['family'],
            'priority': config_data['priority'],
            'started_at': sampleinfo_data['date'],
            'version': sampleinfo_data['version'],
            'out_dir': config_data['out_dir'],
            'config_path': config_data['config_path'],
            'type': cls._get_analysis_type(analysis_types),
        }

        sacct_data, last_job_end = cls._parse_sacct(sacct_jobs, jobs_count=jobs)
        run_data.update(sacct_data)

        run_data['status'] = cls.get_status(sampleinfo_data['is_finished'],
                                            len(run_data['failed_jobs']))
        if run_data['status'] == 'completed':
            run_data['completed_at'] = last_job_end

        return run_data

    @staticmethod
    def _parse_sacct(sacct_jobs: List[dict], jobs_count: int=None):
        """Parse out info from Sacct log."""
        failed_jobs = sacct_api.filter_jobs(sacct_jobs, failed=True)
        completed_jobs = [job for job in sacct_jobs if job['is_completed']]
        last_job_end = completed_jobs[-1]['end'] if len(completed_jobs) > 0 else None
        data = {
            'jobs': jobs_count,
            'completed_jobs': len(completed_jobs),
            'progress': (len(completed_jobs) / jobs_count) if jobs_count else None,
            'failed_jobs': [{
                'slurm_id': job['id'],
                'started_at': job['start'],
                'elapsed': job['elapsed'],
                'status': job['state'].lower(),
                'name': job['step'],
                'context': job['context'],
            } for job in failed_jobs]
        }
        return data, last_job_end

    @staticmethod
    def get_status(finished: str, failed_jobs: int) -> str:
        """Determine status of a run."""
        if finished:
            return 'completed'
        elif failed_jobs > 0:
            return 'failed'
        else:
            return 'running'

    @staticmethod
    def _get_analysis_type(analysis_types: List[str]) -> str:
        """Determine the overall analysis type."""
        types_set = set(analysis_types)
        return types_set.pop() if len(types_set) == 1 else 'wgs'

    def build(self, run_data: dict) -> models.Analysis:
        """Build a new Analysis object."""
        existing_run = self.store.find_analysis(family=run_data['family'],
                                                started_at=run_data['started_at'],
                                                status=run_data['status'])
        if existing_run:
            return None

        run_data['user'] = self.store.user(run_data['user'])
        new_failed_jobs = [self.store.Job(**job) for job in run_data['failed_jobs']]
        del run_data['failed_jobs']
        new_run = self.store.Analysis(**run_data)
        new_run.failed_jobs = new_failed_jobs
        return new_run



# -*- coding: utf-8 -*-
import datetime
from pathlib import Path

import pytest

from trailblazer import log, exc
from trailblazer.log import LogAnalysis


@pytest.mark.parametrize('kwargs, expected_status', [
    (dict(finished=False, failed_jobs=0), 'running'),
    (dict(finished=True, failed_jobs=0), 'completed'),
    (dict(finished=False, failed_jobs=3), 'failed'),
    (dict(finished=True, failed_jobs=3), 'completed'),  # allows override!
])
def test_get_status(kwargs, expected_status):
    # GIVEN the inputs of wheter the run is finished and how many jobs have failed
    # WHEN getting the run status...
    run_status = log.LogAnalysis.get_status(**kwargs)
    # THEN it should return the expected status
    assert run_status == expected_status


def test_call(store, log_analysis, files):

    # GIVEN an empty store
    assert store.analyses().count() == 0

    # WHEN adding a new analysis log entry
    with Path(files['config']).open() as config_stream:
        new_run = log_analysis(config_stream, sampleinfo=files['sampleinfo'], sacct=files['sacct'])

    # THEN it should add the analysis log
    assert store.analyses(family=new_run.family).first() == new_run


def test_call_twice(store, log_analysis, files):
    # GIVEN an empty store
    assert store.analyses().count() == 0

    # WHEN adding a new analysis log entry twice
    with Path(files['config']).open() as config_stream:
        log_analysis(config_stream, sampleinfo=files['sampleinfo'], sacct=files['sacct'])
    first_analysis = store.analyses().first()
    with Path(files['config']).open() as config_stream:
        new_run = log_analysis(config_stream, sampleinfo=files['sampleinfo'], sacct=files['sacct'])

    # THEN it should skip adding the duplicate log to the database
    assert new_run is None
    assert store.analyses(family=first_analysis.family).count() == 1
    assert store.analyses(family=first_analysis.family).first() == first_analysis


def test_call_with_missing_files(log_analysis, files):
    # GIVEN missing sampleinfo and existing sacct
    config_path = Path(files['config'])
    sampleinfo_path = Path('/tmp/I_DO_NOT_EXIST.txt')
    sacct_path = Path(files['sacct'])
    assert sampleinfo_path.exists() is False
    assert sacct_path.exists() is True
    # WHEN calling the log analysis API
    # THEN it should raise an exception
    with pytest.raises(exc.MissingFileError):
        with config_path.open() as config_stream:
            log_analysis(config_stream, sampleinfo=sampleinfo_path, sacct=sacct_path)

    # GIVEN existing sampleinfo and missing sacct
    sampleinfo_path = Path(files['sampleinfo'])
    sacct_path = Path('/tmp/I_DO_NOT_EXIST.txt')
    assert sampleinfo_path.exists() is True
    assert sacct_path.exists() is False
    # WHEN calling the log analysis API
    # THEN it should also raise and exception
    with pytest.raises(exc.MissingFileError):
        with config_path.open() as config_stream:
            log_analysis(config_stream, sampleinfo=sampleinfo_path, sacct=sacct_path)


def test_parse_sacct(files_data):
    # GIVEN sacct jobs from a finished analysis
    sacct_jobs = files_data['sacct']
    analysis_end = datetime.datetime(2017, 6, 19, 3, 39, 56)
    # WHEN parsing them for info
    jobs_count = len(sacct_jobs)    # TODO: see code in _call_ in class LogAnalysis for proper
    # jobs_count calculation
    sacct_data, last_job_end = log.LogAnalysis._parse_sacct(sacct_jobs, jobs_count=jobs_count)
    # THEN it should return info about the jobs in general
    assert sacct_data['jobs'] == len(sacct_jobs)
    assert sacct_data['completed_jobs'] == len(sacct_jobs)
    # ... and correctly determine the end of the last job
    assert last_job_end == analysis_end


def test_parse_sacct_all_fails(allfailed_sacct_jobs):
    # GIVEN sacct jobs from analysis with all failed jobs
    # WHEN parsing out info
    sacct_data, last_job_end = log.LogAnalysis._parse_sacct(allfailed_sacct_jobs)
    # THEN the "analysis end" time should be unknown
    assert last_job_end is None


@pytest.mark.parametrize('analysis_types, expected_type', [
    (['wes'], 'wes'),                # single sample and analysis type
    (['wgs', 'wgs', 'wgs'], 'wgs'),  # multiple samples, single type
    (['wes', 'wgs', 'wgs'], 'wgs'),  # multiple samples, multiple types
])
def test_get_analysis_type(analysis_types, expected_type):
    # GIVEN a list of analysis types; wes, wgs, rna
    # WHEN determining the overall type
    analysis_type = log.LogAnalysis._get_analysis_type(analysis_types)
    # THEN it should be correct :-P
    assert analysis_type == expected_type


def test_parse(files_data):
    # GIVEN a finished analysis with all jobs completed
    config_data = files_data['config']
    sampleinfo_data = files_data['sampleinfo']
    sacct_jobs = files_data['sacct']
    # WHEN parsing log-information
    run_data = log.LogAnalysis.parse(config_data, sampleinfo_data, sacct_jobs, jobs=None)
    # THEN it should have parsed out a completed date
    assert run_data['status'] == 'completed'
    assert isinstance(run_data['completed_at'], datetime.datetime)



# -*- coding: utf-8 -*-
"""Test MIP files"""

from trailblazer.mip import files


def test_parse_config(files_raw) -> dict:
    """
    Args:
    files_raw (dict): With dicts from files
    """

    # GIVEN config data of a "sharp" run (not dry run)
    config_raw = files_raw['config']

    # WHEN parsing the MIP output config
    config_data = files.parse_config(config_raw)

    # THEN it should work
    assert isinstance(config_data, dict)

    # THEN it should work and the dry run check should be "no"
    assert config_data['is_dryrun'] is False

    # ... and should include all samples
    assert len(config_raw['analysis_type']) == len(config_data['samples'])

    # Build dict for return data
    config_test_data = {
        'config_path': 'tests/fixtures/family/family_config.yaml',
        'email': 'emilia.ottosson@scilifelab.se',
        'family': 'family',
        'log_path': 'tests/fixtures/family/mip.pl_2017-06-17T12:11:42.log',
        'sampleinfo_path': ('/mnt/hds/proj/bioinfo/MIP_ANALYSIS/' +
                            'customers/cust003/family/analysis/' +
                            'family_qc_sample_info.yaml'),
        'out_dir': ('/mnt/hds/proj/bioinfo/MIP_ANALYSIS/customers/' +
                    'cust003/family/analysis'),
        'priority': 'normal',
        }

    # Check returns from def
    for key, value in config_test_data.items():
        assert config_data[key] == value

def test_parse_sampleinfo(files_raw):
    """
    Args:
    files_raw (dict): With dicts from files
    """

    # GIVEN sampleinfo input from a finished analysis
    sampleinfo_raw = files_raw['sampleinfo']

    # WHEN parsing the MIP output sampleinfo
    sampleinfo_data = files.parse_sampleinfo(sampleinfo_raw)

    # THEN it should work
    assert isinstance(sampleinfo_data, dict)

    # THEN it should mark the run as "finished"
    assert sampleinfo_data['is_finished'] is True

    ### More in-depth testing
    ## Family data
    # Build dict for family return data
    sampleinfo_test_data = {
        'family': 'family',
        'genome_build': 'GRCh37',
        'pedigree_path': '/path_to/family/family.fam',
        'qcmetrics_path': ('/path_to/family/family_qc_metrics.yaml'),
        'svdb_outpath': 'path/to/118_sorted_md_brecal_comb_SV.vcf',
        'version': 'v6.0.0',
        }

    # Check returns from def
    for key, value in sampleinfo_test_data.items():
        assert sampleinfo_data[key] == value

    ## Sample data
    # Build dict for sample return data
    sampleinfo_test_sample_data = {
        'bam': '/path_to/sample/bwa/sample_lanes_7777_sorted_md_brecal.bam',
        'chanjo_sexcheck': ('/path_to/sample/bwa/coveragereport/' +
                            'sample_lanes_7777_sorted_md_brecal.sexcheck'),
        'id': 'sample',
        'sambamba': ('/path_to/sample/bwa/coveragereport/sample_lanes_7777' +
                     '_sorted_md_brecal_coverage.bed'),
        'sex': 'female',
        'subsample_mt': ('/path_to/sample/bwa/ADM1059A4_lanes_8_sorted_md' +
                         '_brecal_subsample_MT.bam'),
        'vcf2cytosure': ('/path_to/sample/bwa/vcf2cytosure/ADM1059A4_sorted' +
                         '_md_brecal_comb_SV.cgh'),
        }

    # Check returns from def
    for key, value in sampleinfo_test_sample_data.items():
        for sample_data in sampleinfo_data['samples']:
            assert sample_data[key] == value

    assert len(sampleinfo_raw['analysis_type']) == len(sampleinfo_data['samples'])

    ## Snv data
    # Build dict for snv return data
    sampleinfo_test_snv_data = {
        'clinical_vcf': ('/path_to/family_sorted_md_brecal_gvcf_vrecal_comb' +
                         '_vt_vep_parsed_snpeff_ranked_BOTH.selected.vcf.gz'),
        'research_vcf': ('/path_to/family_sorted_md_brecal_gvcf_vrecal_comb' +
                         '_vt_vep_parsed_snpeff_ranked_BOTH.vcf.gz'),
        'bcf': '/path_to/family_sorted_md_brecal_gvcf_vrecal_comb_BOTH.bcf',
        'gbcf': '/path_to/gatk/family_sorted_md_brecal_gvcf_BOTH.bcf'
        }

    # Check returns from def
    for key, value in sampleinfo_test_snv_data.items():
        assert sampleinfo_data['snv'][key] == value

    ## SV data
    # Build dict for sv return data
    sampleinfo_test_sv_data = {
        'clinical_vcf': ('/path_to/family_sorted_md_brecal_comb_vep_' +
                         'parsed_ranked_SV.selected.vcf.gz'),
        'research_vcf': ('/path_to/family_sorted_md_brecal_comb_vep_' +
                         'parsed_ranked_SV.vcf.gz'),
        'bcf': '/path_to/family_sorted_md_brecal_comb_SV.bcf',
        'merged': 'path/to/118_sorted_md_brecal_comb_SV.vcf',
        }

    # Check returns from def
    for key, value in sampleinfo_test_sv_data.items():
        assert sampleinfo_data['sv'][key] == value

    ## Peddy data
    # Build dict for peddy return data
    sampleinfo_test_peddy_data = {
        'ped_check': '/path_to/118.ped_check.csv',
        'ped': '/path_to/118.peddy.ped',
        'sex_check': '/path_to/118.sex_check.csv',
        }

    # Check returns from def
    for key, value in sampleinfo_test_peddy_data.items():
        assert sampleinfo_data['peddy'][key] == value

def test_parse_qcmetrics(files_raw):
    """
    Args:
    files_raw (dict): With dicts from files
    """

    # GIVEN qc metrics input from an analysis
    qcmetrics_raw = files_raw['qcmetrics']

    # WHEN parsing it
    qcmetrics_data = files.parse_qcmetrics(qcmetrics_raw)

    # THEN it should work ;-)
    assert isinstance(qcmetrics_data, dict)

    ## Version data
    # Build dict for version return data
    qcmetrics_test_version_data = {
        'freebayes': 'v1.0.2',
        'gatk': 3.6,
        'manta': '1.0.3',
        'bcftools': '1.3.1+htslib-1.3.1',
        'vep': 'v87',
        }

    # Check returns from def
    for key, value in qcmetrics_test_version_data.items():
        assert qcmetrics_data['versions'][key] == value

    ## Sample data
    # Build dict for sample return data
    qcmetrics_test_sample_data = {
        'at_dropout': 0.126963,
        'duplicates': 0.132719986683768,
        'id': 'sample',
        'insert_size_standard_deviation': 89.871783,
        'gc_dropout': 3.956909,
        'mapped': 0.9850732625744484,
        'median_insert_size': 413,
        'plink_sex': 'female',
        'predicted_sex': 'female',
        'reads': 949878168,
        'strand_balance': 0.499558,
        'target_coverage': 37.428611,
        }

    # Check returns from def
    for key, value in qcmetrics_test_sample_data.items():
        for sample_data in qcmetrics_data['samples']:
            assert sample_data[key] == value

    ## Sample coverage data
    # Build dict for sample coverage return data
    qcmetrics_test_sample_cov_data = {
        10: 0.991187,
        20: 0.984713,
        50: 0.063229,
        100: 0.000372,
        }
    # Check returns from def
    for key, value in qcmetrics_test_sample_cov_data.items():
        for sample_data in qcmetrics_data['samples']:
            assert sample_data['completeness_target'][key] == value



# -*- coding: utf-8 -*-
"""Parse the MIP config file."""
from typing import TextIO
import csv

PED_SEX_MAP = {1: 'male', 2: 'female', 0: 'unknown'}


def parse_config(data: dict) -> dict:
    """Parse MIP config file.

    Args:
        data (dict): raw YAML input from MIP analysis config file

    Returns:
        dict: parsed data
    """
    return {
        'email': data.get('email'),
        'family': data['family_id'],
        'samples': [{
            'id': sample_id,
            'type': analysis_type,
        } for sample_id, analysis_type in data['analysis_type'].items()],
        'config_path': data['config_file_analysis'],
        'is_dryrun': True if 'dry_run_all' in data else False,
        'log_path': data['log_file'],
        'out_dir': data['outdata_dir'],
        'priority': data['slurm_quality_of_service'],
        'sampleinfo_path': data['sample_info_file'],
    }


def parse_sampleinfo(data: dict) -> dict:
    """Parse MIP sample info file.

    Args:
        data (dict): raw YAML input from MIP qc sample info file

    Returns:
        dict: parsed data
    """

    genome_build = data['human_genome_build']
    genome_build_str = f"{genome_build['source']}{genome_build['version']}"
    if 'svdb' in data['program']:
        svdb_outpath = (f"{data['program']['svdb']['path']}")
    else:
        svdb_outpath = ''
    outdata = {
        'date': data['analysis_date'],
        'family': data['family'],
        'genome_build': genome_build_str,
        'rank_model_version': data['program']['genmod']['rank_model']['version'],
        'is_finished': True if data['analysisrunstatus'] == 'finished' else False,
        'pedigree_path': data['pedigree_minimal'],
        'peddy': {
            'ped': (data['program']['peddy']['peddy']['path'] if
                    'peddy' in data['program'] else None),
            'ped_check': (data['program']['peddy']['ped_check']['path'] if
                          'peddy' in data['program'] else None),
            'sex_check': (data['program']['peddy']['sex_check']['path'] if
                          'peddy' in data['program'] else None),
        },
        'qcmetrics_path': data['program']['qccollect']['path'],
        'samples': [],
        'snv': {
            'bcf': data['most_complete_bcf']['path'],
            'clinical_vcf': data['vcf_binary_file']['clinical']['path'],
            'gbcf': data['gbcf_file']['path'],
            'research_vcf': data['vcf_binary_file']['research']['path'],
        },
        'svdb_outpath': svdb_outpath,
        'sv': {
            'bcf': data.get('sv_bcf_file', {}).get('path'),
            'clinical_vcf': (data['sv_vcf_binary_file']['clinical']['path'] if
                             'sv_vcf_binary_file' in data else None),
            'merged': svdb_outpath,
            'research_vcf': (data['sv_vcf_binary_file']['research']['path'] if
                             'sv_vcf_binary_file' in data else None),
        },
        'version': data['mip_version'],
    }

    for sample_id, sample_data in data['sample'].items():
        sample = {
            'id': sample_id,
            'bam': sample_data['most_complete_bam']['path'],
            'sambamba': list(sample_data['program']['sambamba_depth'].values())[0]['path'],
            'sex': sample_data['sex'],
            # subsample mt is only for wgs data
            'subsample_mt': (list(sample_data['program']['samtools_subsample_mt'].values())[0]['path'] if
                             'samtools_subsample_mt' in sample_data['program'] else None),
            'vcf2cytosure': list(sample_data['program']['vcf2cytosure'].values())[0]['path'],
        }
        chanjo_sexcheck = list(sample_data['program']['chanjo_sexcheck'].values())[0]
        sample['chanjo_sexcheck'] = chanjo_sexcheck['path']
        outdata['samples'].append(sample)

    return outdata


def parse_qcmetrics(metrics: dict) -> dict:
    """Parse MIP qc metrics file.
    Args:
        metrics (dict): raw YAML input from MIP qc metrics file

    Returns:
        dict: parsed data
    """
    data = {
        'versions': {
            'freebayes': metrics['program']['freebayes']['version'],
            'gatk': metrics['program']['gatk']['version'],
            'manta': metrics['program'].get('manta', {}).get('version'),
            'bcftools': metrics['program']['bcftools']['version'],
            'vep': metrics['program']['varianteffectpredictor']['version'],
        },
        'samples': [],
    }

    plink_samples = {}
    plink_sexcheck = metrics['program'].get('plink_sexcheck', {}).get('sample_sexcheck')
    if isinstance(plink_sexcheck, str):
        sample_id, sex_number = plink_sexcheck.strip().split(':', 1)
        plink_samples[sample_id] = PED_SEX_MAP.get(int(sex_number))
    elif isinstance(plink_sexcheck, list):
        for sample_raw in plink_sexcheck:
            sample_id, sex_number = sample_raw.split(':', 1)
            plink_samples[sample_id] = PED_SEX_MAP.get(int(sex_number))

    for sample_id, sample_metrics in metrics['sample'].items():

        ## Bam stats metrics
        bam_stats = [values['bamstats'] for key, values in sample_metrics.items()
                     if key[:-1].endswith('.lane')]
        total_reads = sum(int(bam_stat['raw_total_sequences']) for bam_stat in bam_stats)
        total_mapped = sum(int(bam_stat['reads_mapped']) for bam_stat in bam_stats)

        ## Picard metrics
        main_key = [key for key in sample_metrics.keys() if '_lanes_' in key][0]

        hs_metrics = sample_metrics[main_key]['collecthsmetrics']['header']['data']
        multiple_inst_metrics = sample_metrics[main_key]['collectmultiplemetricsinsertsize']['header']['data']
        multiple_metrics = sample_metrics[main_key]['collectmultiplemetrics']['header']['pair']

        sample_data = {
            'at_dropout': hs_metrics['AT_DROPOUT'],
            'completeness_target': {
                10: hs_metrics['PCT_TARGET_BASES_10X'],
                20: hs_metrics['PCT_TARGET_BASES_20X'],
                50: hs_metrics['PCT_TARGET_BASES_50X'],
                100: hs_metrics['PCT_TARGET_BASES_100X'],
            },
            'duplicates': float(sample_metrics[main_key]['markduplicates']['fraction_duplicates']),
            'gc_dropout': hs_metrics['GC_DROPOUT'],
            'id': sample_id,
            'median_insert_size':  multiple_inst_metrics['MEDIAN_INSERT_SIZE'],
            'mapped': total_mapped / total_reads,
            'plink_sex': plink_samples.get(sample_id),
            'predicted_sex': sample_metrics[main_key]['chanjo_sexcheck']['gender'],
            'reads': total_reads,
            'insert_size_standard_deviation': float(multiple_inst_metrics['STANDARD_DEVIATION']),
            'strand_balance': float(multiple_metrics['STRAND_BALANCE']),
            'target_coverage': float(hs_metrics['MEAN_TARGET_COVERAGE']),
        }
        data['samples'].append(sample_data)
    return data


def parse_peddy_sexcheck(handle: TextIO):
    """Parse Peddy sexcheck output."""
    data = {}
    samples = csv.DictReader(handle)
    for sample in samples:
        data[sample['sample_id']] = {
            'predicted_sex': sample['predicted_sex'],
            'het_ratio': float(sample['het_ratio']),
            'error': True if sample['error'] == 'True' else False,
        }
    return data


def parse_chanjo_sexcheck(handle: TextIO):
    """Parse Chanjo sex-check output."""
    samples = csv.DictReader(handle, delimiter='\t')
    for sample in samples:
        return {
            'predicted_sex': sample['sex'],
            'x_coverage': float(sample['#X_coverage']),
            'y_coverage': float(sample['Y_coverage']),
        }



# -*- coding: utf-8 -*-


def test_build_command(mip_cli):
    # GIVEN just a config
    mip_config = 'tests/fixtures/global_config.yaml'
    # WHEN building the MIP command
    mip_command = mip_cli.build_command(mip_config)
    # THEN it should use the correct options
    assert mip_command == ['perl', mip_cli.script, '--config_file', mip_config]


def test_build_command_with_default(mip_cli):
    # GIVEN a supported option with a default
    option = '--dry_run_all'
    default = '2'
    # WHEN building a MIP command
    mip_command = mip_cli.build_command('test/mip.conf', dryrun=True)
    # THEN it should pick the right command and choose the default
    assert option in mip_command
    assert mip_command[mip_command.index(option) + 1] == default


def test_build_command_without_default(mip_cli):
    # GIVEN a supported option without a default
    option = '--slurm_quality_of_service'
    value = 'high'
    # WHEN building a MIP command
    mip_command = mip_cli.build_command('test/mip.conf', priority=value)
    # THEN it should pick the right command and choose the default
    assert option in mip_command
    assert mip_command[mip_command.index(option) + 1] == value
